@@ -91,7 +91,7 @@ class NotificationType(models.TextChoices):
         "Upwell structure went low power"
     )
     STRUCTURE_UNANCHORING = "StructureUnanchoring", _("Upwell structure unanchoring")
-    STRUCTURE_FUEL_ALERT = "FuelAlert", _("Upwell structure fuel alert")
+    STRUCTURE_FUEL_ALERT = "StructureFuelAlert", _("Upwell structure fuel alert")
     STRUCTURE_REFUELED_EXTRA = "StructureRefueledExtra", _("Upwell structure refueled")
     STRUCTURE_JUMP_FUEL_ALERT = "StructureJumpFuelAlert", _(
         "Upwell structure jump fuel alert"
@@ -188,11 +188,15 @@ class NotificationType(models.TextChoices):
 
     @classproperty
     def esi_notifications(cls) -> set:
-        my_set = set(cls.values)
-        my_set.discard(cls.STRUCTURE_JUMP_FUEL_ALERT)
-        my_set.discard(cls.STRUCTURE_REFUELED_EXTRA)
-        my_set.discard(cls.TOWER_REFUELED_EXTRA)
-        return my_set
+        return set(cls.values) - cls.generated_notifications
+
+    @classproperty
+    def generated_notifications(cls) -> set:
+        return {
+            cls.STRUCTURE_JUMP_FUEL_ALERT,
+            cls.STRUCTURE_REFUELED_EXTRA,
+            cls.TOWER_REFUELED_EXTRA,
+        }
 
     @classproperty
     def webhook_defaults(cls) -> list:
@@ -218,6 +222,7 @@ class NotificationType(models.TextChoices):
 
     @classproperty
     def relevant_for_timerboard(cls) -> set:
+        """Notification types that can create timers."""
         return {
             cls.STRUCTURE_LOST_SHIELD,
             cls.STRUCTURE_LOST_ARMOR,
@@ -229,6 +234,7 @@ class NotificationType(models.TextChoices):
 
     @classproperty
     def relevant_for_alliance_level(cls) -> set:
+        """Notification types that requre the alliance level flag."""
         return {
             # sov
             cls.SOV_ENTOSIS_CAPTURE_STARTED,
@@ -251,6 +257,7 @@ class NotificationType(models.TextChoices):
 
     @classproperty
     def relevant_for_moonmining(cls) -> set:
+        """Notification types about moon mining."""
         return {
             cls.MOONMINING_EXTRACTION_STARTED,
             cls.MOONMINING_EXTRACTION_CANCELLED,
@@ -259,8 +266,40 @@ class NotificationType(models.TextChoices):
             cls.MOONMINING_AUTOMATIC_FRACTURE,
         }
 
+    @classproperty
+    def structure_related(cls) -> set:
+        """Notification types that are related to a structure."""
+        return {
+            cls.STRUCTURE_ONLINE,
+            cls.STRUCTURE_FUEL_ALERT,
+            cls.STRUCTURE_JUMP_FUEL_ALERT,
+            cls.STRUCTURE_REFUELED_EXTRA,
+            cls.STRUCTURE_SERVICES_OFFLINE,
+            cls.STRUCTURE_WENT_LOW_POWER,
+            cls.STRUCTURE_WENT_HIGH_POWER,
+            cls.STRUCTURE_UNANCHORING,
+            cls.STRUCTURE_UNDER_ATTACK,
+            cls.STRUCTURE_LOST_SHIELD,
+            cls.STRUCTURE_LOST_ARMOR,
+            cls.STRUCTURE_DESTROYED,
+            cls.OWNERSHIP_TRANSFERRED,
+            cls.STRUCTURE_ANCHORING,
+            cls.MOONMINING_EXTRACTION_STARTED,
+            cls.MOONMINING_EXTRACTION_FINISHED,
+            cls.MOONMINING_AUTOMATIC_FRACTURE,
+            cls.MOONMINING_EXTRACTION_CANCELLED,
+            cls.MOONMINING_LASER_FIRED,
+            cls.STRUCTURE_REINFORCE_CHANGED,
+            cls.ORBITAL_ATTACKED,
+            cls.ORBITAL_REINFORCED,
+            cls.TOWER_ALERT_MSG,
+            cls.TOWER_RESOURCE_ALERT_MSG,
+            cls.TOWER_REFUELED_EXTRA,
+        }
+
     @classproperty
     def relevant_for_forwarding(cls) -> set:
+        """Notification types that are forwarded to Discord."""
         my_set = set(cls.values_enabled)
         # if STRUCTURES_NOTIFICATION_DISABLE_ESI_FUEL_ALERTS:
         #     my_set.discard(cls.STRUCTURE_FUEL_ALERT)
@@ -367,6 +406,7 @@ class EveEntity(models.Model):
             self.__class__.__name__, self.id, self.get_category_display(), self.name
         )
 
+    @property
     def profile_url(self) -> str:
         """Returns link to website with profile info about this entity."""
         if self.category == self.Category.CORPORATION:
@@ -399,7 +439,7 @@ class Notification(models.Model):
     """
 
     HTTP_CODE_TOO_MANY_REQUESTS = 429
-    GENERATED_NOTIFICATION_ID = 999999999999
+    TEMPORARY_NOTIFICATION_ID = 999999999999
 
     # event type structure map
     MAP_CAMPAIGN_EVENT_2_TYPE_ID = {
@@ -420,18 +460,12 @@ class Notification(models.Model):
         help_text="Corporation that received this notification",
     )
 
-    sender = models.ForeignKey(EveEntity, on_delete=models.CASCADE)
-    timestamp = models.DateTimeField(db_index=True)
-    notif_type = models.CharField(
-        max_length=100,
-        default="",
-        db_index=True,
-        verbose_name="type",
-        help_text="type of this notification as reported by ESI",
-    )
-    text = models.TextField(
-        null=True, default=None, blank=True, help_text="Notification details in YAML"
+    created = models.DateTimeField(
+        null=True,
+        default=None,
+        help_text="Date when this notification was first received from ESI",
     )
+
     is_read = models.BooleanField(
         null=True,
         default=None,
@@ -449,11 +483,23 @@ class Notification(models.Model):
     last_updated = models.DateTimeField(
         help_text="Date when this notification has last been updated from ESI"
     )
-    created = models.DateTimeField(
-        null=True,
-        default=None,
-        help_text="Date when this notification was first received from ESI",
+    notif_type = models.CharField(
+        max_length=100,
+        default="",
+        db_index=True,
+        verbose_name="type",
+        help_text="type of this notification as reported by ESI",
     )
+    sender = models.ForeignKey(EveEntity, on_delete=models.CASCADE)
+    structures = models.ManyToManyField(
+        Structure,
+        related_name="notifications",
+        help_text="Structures this notification is about (if any)",
+    )
+    text = models.TextField(
+        null=True, default=None, blank=True, help_text="Notification details in YAML"
+    )
+    timestamp = models.DateTimeField(db_index=True)
 
     objects = NotificationManager()
 
@@ -472,13 +518,14 @@ class Notification(models.Model):
         )
 
     def save(self, *args, **kwargs) -> None:
-        if not self.is_generated:
-            super().save(*args, **kwargs)  # generated notifications can not be saved
+        if self.is_temporary:
+            raise ValueError("Temporary notifications can not be saved")
+        super().save(*args, **kwargs)
 
     @property
-    def is_generated(self) -> bool:
-        """True when this notification was generated by Structures."""
-        return self.notification_id == self.GENERATED_NOTIFICATION_ID
+    def is_temporary(self) -> bool:
+        """True when this notification is temporary."""
+        return self.notification_id == self.TEMPORARY_NOTIFICATION_ID
 
     @property
     def is_alliance_level(self) -> bool:
@@ -495,12 +542,17 @@ class Notification(models.Model):
         """Whether this notification can have a timer."""
         return self.notif_type in NotificationType.relevant_for_timerboard
 
+    @property
+    def is_structure_related(self) -> bool:
+        """Wheater this notification related to a structure."""
+        return self.notif_type in NotificationType.structure_related
+
     # @classmethod
     # def get_all_types(cls) -> Set[int]:
     #     """returns a set with all supported notification types"""
     #     return {x[0] for x in NotificationType.choices}
 
-    def get_parsed_text(self) -> dict:
+    def parsed_text(self) -> dict:
         """Returns the notifications's text as dict."""
         return yaml.safe_load(self.text)
 
@@ -511,7 +563,7 @@ class Notification(models.Model):
             NotificationType.ORBITAL_ATTACKED,
             NotificationType.STRUCTURE_UNDER_ATTACK,
         ]:
-            parsed_text = self.get_parsed_text()
+            parsed_text = self.parsed_text()
             corporation_id = None
             if self.notif_type == NotificationType.STRUCTURE_UNDER_ATTACK:
                 if (
@@ -541,6 +593,75 @@ class Notification(models.Model):
             and not self.owner.is_alliance_main
         )
 
+    def update_related_structures(self) -> bool:
+        """Update related structure for this notification.
+
+        Returns True if structures where updated, else False.
+        """
+        structures_qs = self.calc_related_structures()
+        self.structures.clear()
+        if structures_qs.exists():
+            objs = [obj for obj in structures_qs.all()]
+            self.structures.add(*objs)
+            return True
+        return False
+
+    def calc_related_structures(self) -> models.QuerySet[Structure]:
+        """Identify structures this notification is related to.
+
+        Returns:
+        - structures if any found or empty list if there are no related structures
+        """
+        parsed_text = self.parsed_text()
+        if not parsed_text:
+            return Structure.objects.none()
+        if self.notif_type in {
+            NotificationType.STRUCTURE_ONLINE,
+            NotificationType.STRUCTURE_FUEL_ALERT,
+            NotificationType.STRUCTURE_JUMP_FUEL_ALERT,
+            NotificationType.STRUCTURE_REFUELED_EXTRA,
+            NotificationType.STRUCTURE_SERVICES_OFFLINE,
+            NotificationType.STRUCTURE_WENT_LOW_POWER,
+            NotificationType.STRUCTURE_WENT_HIGH_POWER,
+            NotificationType.STRUCTURE_UNANCHORING,
+            NotificationType.STRUCTURE_UNDER_ATTACK,
+            NotificationType.STRUCTURE_LOST_SHIELD,
+            NotificationType.STRUCTURE_LOST_ARMOR,
+            NotificationType.STRUCTURE_DESTROYED,
+            NotificationType.OWNERSHIP_TRANSFERRED,
+            NotificationType.STRUCTURE_ANCHORING,
+            NotificationType.MOONMINING_EXTRACTION_STARTED,
+            NotificationType.MOONMINING_EXTRACTION_FINISHED,
+            NotificationType.MOONMINING_AUTOMATIC_FRACTURE,
+            NotificationType.MOONMINING_EXTRACTION_CANCELLED,
+            NotificationType.MOONMINING_LASER_FIRED,
+        }:
+            structure_id = parsed_text.get("structureID")
+            return Structure.objects.filter(id=structure_id)
+        elif self.notif_type == NotificationType.STRUCTURE_REINFORCE_CHANGED:
+            structure_ids = [
+                structure_info[0] for structure_info in parsed_text["allStructureInfo"]
+            ]
+            return Structure.objects.filter(id__in=structure_ids)
+
+        elif self.notif_type in {
+            NotificationType.ORBITAL_ATTACKED,
+            NotificationType.ORBITAL_REINFORCED,
+        }:
+            return Structure.objects.filter(
+                eve_planet_id=parsed_text["planetID"], eve_type_id=parsed_text["typeID"]
+            )
+
+        elif self.notif_type in {
+            NotificationType.TOWER_ALERT_MSG,
+            NotificationType.TOWER_RESOURCE_ALERT_MSG,
+            NotificationType.TOWER_REFUELED_EXTRA,
+        }:
+            return Structure.objects.filter(
+                eve_moon_id=parsed_text["moonID"], eve_type_id=parsed_text["typeID"]
+            )
+        return Structure.objects.none()
+
     def _generate_embed(
         self, language_code: str
     ) -> Tuple[dhooks_lite.Embed, Webhook.PingType]:
@@ -562,7 +683,7 @@ class Notification(models.Model):
         if (
             has_auth_timers or has_structure_timers
         ) and self.notif_type in NotificationType.relevant_for_timerboard:
-            parsed_text = self.get_parsed_text()
+            parsed_text = self.parsed_text()
             try:
                 with translation.override(STRUCTURES_DEFAULT_LANGUAGE):
                     if self.notif_type in [
@@ -637,8 +758,8 @@ class Notification(models.Model):
 
         if has_structure_timers:
             timer_map = {
-                NotificationType.STRUCTURE_LOST_SHIELD: Timer.TYPE_ARMOR,
-                NotificationType.STRUCTURE_LOST_ARMOR: Timer.TYPE_HULL,
+                NotificationType.STRUCTURE_LOST_SHIELD: Timer.Type.ARMOR,
+                NotificationType.STRUCTURE_LOST_ARMOR: Timer.Type.HULL,
             }
             eve_solar_system, _ = EveSolarSystem2.objects.get_or_create_esi(
                 id=structure_obj.eve_solar_system_id
@@ -647,15 +768,15 @@ class Notification(models.Model):
                 id=structure_obj.eve_type_id
             )
             visibility = (
-                Timer.VISIBILITY_CORPORATION
+                Timer.Visibility.CORPORATION
                 if STRUCTURES_TIMERS_ARE_CORP_RESTRICTED
-                else Timer.VISIBILITY_UNRESTRICTED
+                else Timer.Visibility.UNRESTRICTED
             )
             Timer.objects.create(
                 eve_solar_system=eve_solar_system,
                 structure_type=structure_type,
                 timer_type=timer_map.get(self.notif_type),
-                objective=Timer.OBJECTIVE_FRIENDLY,
+                objective=Timer.Objective.FRIENDLY,
                 date=eve_time,
                 eve_corporation=self.owner.corporation,
                 eve_alliance=self.owner.corporation.alliance,
@@ -707,15 +828,15 @@ class Notification(models.Model):
                 id=self.type_id_from_event_type(parsed_text["campaignEventType"])
             )
             visibility = (
-                Timer.VISIBILITY_CORPORATION
+                Timer.Visibility.CORPORATION
                 if STRUCTURES_TIMERS_ARE_CORP_RESTRICTED
-                else Timer.VISIBILITY_UNRESTRICTED
+                else Timer.Visibility.UNRESTRICTED
             )
             Timer.objects.create(
                 eve_solar_system=eve_solar_system,
                 structure_type=structure_type,
-                timer_type=Timer.TYPE_FINAL,
-                objective=Timer.OBJECTIVE_FRIENDLY,
+                timer_type=Timer.Type.FINAL,
+                objective=Timer.Objective.FRIENDLY,
                 date=eve_time,
                 eve_corporation=self.owner.corporation,
                 eve_alliance=self.owner.corporation.alliance,
@@ -756,15 +877,15 @@ class Notification(models.Model):
                 id=EveTypeId.CUSTOMS_OFFICE
             )
             visibility = (
-                Timer.VISIBILITY_CORPORATION
+                Timer.Visibility.CORPORATION
                 if STRUCTURES_TIMERS_ARE_CORP_RESTRICTED
-                else Timer.VISIBILITY_UNRESTRICTED
+                else Timer.Visibility.UNRESTRICTED
             )
             Timer.objects.create(
                 eve_solar_system=eve_solar_system,
                 structure_type=structure_type,
-                timer_type=Timer.TYPE_FINAL,
-                objective=Timer.OBJECTIVE_FRIENDLY,
+                timer_type=Timer.Type.FINAL,
+                objective=Timer.Objective.FRIENDLY,
                 date=eve_time,
                 location_details=planet.name,
                 eve_corporation=self.owner.corporation,
@@ -803,9 +924,9 @@ class Notification(models.Model):
                 id=parsed_text["structureTypeID"]
             )
             visibility = (
-                Timer.VISIBILITY_CORPORATION
+                Timer.Visibility.CORPORATION
                 if STRUCTURES_TIMERS_ARE_CORP_RESTRICTED
-                else Timer.VISIBILITY_UNRESTRICTED
+                else Timer.Visibility.UNRESTRICTED
             )
         else:
             eve_solar_system = None
@@ -834,15 +955,15 @@ class Notification(models.Model):
                     id=parsed_text["structureTypeID"]
                 )
                 visibility = (
-                    Timer.VISIBILITY_CORPORATION
+                    Timer.Visibility.CORPORATION
                     if STRUCTURES_TIMERS_ARE_CORP_RESTRICTED
-                    else Timer.VISIBILITY_UNRESTRICTED
+                    else Timer.Visibility.UNRESTRICTED
                 )
                 Timer.objects.create(
                     eve_solar_system=eve_solar_system,
                     structure_type=structure_type,
-                    timer_type=Timer.TYPE_MOONMINING,
-                    objective=Timer.OBJECTIVE_FRIENDLY,
+                    timer_type=Timer.Type.MOONMINING,
+                    objective=Timer.Objective.FRIENDLY,
                     date=eve_time,
                     location_details=moon.name,
                     eve_corporation=self.owner.corporation,
@@ -863,7 +984,7 @@ class Notification(models.Model):
             ).order_by("-timestamp")
 
             for notification in notifications_qs:
-                parsed_text_2 = notification.get_parsed_text()
+                parsed_text_2 = notification.parsed_text()
                 my_structure_type_id = parsed_text_2["structureTypeID"]
                 if my_structure_type_id == parsed_text["structureTypeID"]:
                     eve_time = ldap_time_2_datetime(parsed_text_2["readyTime"])
@@ -885,10 +1006,10 @@ class Notification(models.Model):
                         timer_query = Timer.objects.filter(
                             eve_solar_system=eve_solar_system,
                             structure_type=structure_type,
-                            timer_type=Timer.TYPE_MOONMINING,
+                            timer_type=Timer.Type.MOONMINING,
                             location_details=moon.name,
                             date=eve_time,
-                            objective=Timer.OBJECTIVE_FRIENDLY,
+                            objective=Timer.Objective.FRIENDLY,
                             eve_corporation=self.owner.corporation,
                             eve_alliance=self.owner.corporation.alliance,
                             visibility=visibility,
@@ -926,20 +1047,34 @@ class Notification(models.Model):
         """
         if self.filter_for_npc_attacks() or self.filter_for_alliance_level():
             return None
-        webhooks = self.owner.webhooks.filter(
-            notification_types__contains=self.notif_type, is_active=True
-        )
-        if not webhooks.exists():
+        webhooks_qs = self.relevant_webhooks()
+        if not webhooks_qs.exists():
             return None
         if ping_type_override:
             self.ping_type_override = ping_type_override
         if use_color_override:
             self.color_override = color_override
         success = True
-        for webhook in webhooks:
+        for webhook in webhooks_qs:
             success &= self.send_to_webhook(webhook)
         return success
 
+    def relevant_webhooks(self) -> models.QuerySet:
+        """Determine relevant webhooks matching this notification type."""
+        if not self.is_structure_related:
+            structures_qs = None
+        else:
+            structures_qs = self.calc_related_structures()
+        if structures_qs and structures_qs.filter(webhooks__isnull=False).count() == 1:
+            webhooks_qs = structures_qs.first().webhooks.filter(
+                notification_types__contains=self.notif_type, is_active=True
+            )
+        else:
+            webhooks_qs = self.owner.webhooks.filter(
+                notification_types__contains=self.notif_type, is_active=True
+            )
+        return webhooks_qs
+
     def send_to_webhook(self, webhook: Webhook) -> bool:
         """Sends this notification to the configured webhook.
 
@@ -981,12 +1116,9 @@ class Notification(models.Model):
 
         username, avatar_url = self._gen_avatar()
         success = webhook.send_message(
-            content=content,
-            embeds=[embed],
-            username=username,
-            avatar_url=avatar_url,
+            content=content, embeds=[embed], username=username, avatar_url=avatar_url
         )
-        if success:
+        if success and not self.is_temporary:
             self.is_sent = True
             self.save()
 
@@ -1043,6 +1175,7 @@ class Notification(models.Model):
                 data = {
                     "moonID": structure.eve_moon_id,
                     "typeID": structure.eve_type_id,
+                    "structureID": structure.id,
                 }
             else:
                 raise ValueError("text property not provided and can not be generated.")
@@ -1052,7 +1185,7 @@ class Notification(models.Model):
                 eve_entity_id=EveCorporationId.DED
             )
             kwargs["sender"] = sender
-        kwargs["notification_id"] = cls.GENERATED_NOTIFICATION_ID
+        kwargs["notification_id"] = cls.TEMPORARY_NOTIFICATION_ID
         kwargs["owner"] = structure.owner
         kwargs["notif_type"] = notif_type
         return cls(**kwargs)
@@ -1247,10 +1380,10 @@ class FuelAlert(BaseFuelAlert):
     """A generated notification alerting about fuel getting low in structures."""
 
     structure = models.ForeignKey(
-        "Structure", on_delete=models.CASCADE, related_name="structure_fuel_alerts"
+        Structure, on_delete=models.CASCADE, related_name="structure_fuel_alerts"
     )
     config = models.ForeignKey(
-        "FuelAlertConfig",
+        FuelAlertConfig,
         on_delete=models.CASCADE,
         related_name="structure_fuel_alerts",
     )
@@ -1291,10 +1424,10 @@ class JumpFuelAlert(BaseFuelAlert):
     """A generated notification alerting about jump fuel getting low."""
 
     structure = models.ForeignKey(
-        "Structure", on_delete=models.CASCADE, related_name="jump_fuel_alerts"
+        Structure, on_delete=models.CASCADE, related_name="jump_fuel_alerts"
     )
     config = models.ForeignKey(
-        "JumpFuelAlertConfig", on_delete=models.CASCADE, related_name="jump_fuel_alerts"
+        JumpFuelAlertConfig, on_delete=models.CASCADE, related_name="jump_fuel_alerts"
     )
 
     def __str__(self) -> str:
