@@ -20,10 +20,19 @@ from app_utils.views import bootstrap_label_html
 
 from .. import __title__
 from ..app_settings import STRUCTURES_FEATURE_REFUELED_NOTIFICIATIONS
-from ..constants import EveGroupId, EveTypeId
+from ..constants import EveCategoryId, EveGroupId, EveTypeId
+from ..core import starbases
 from ..helpers.general import datetime_almost_equal, hours_until_deadline
 from ..managers import StructureManager, StructureTagManager
-from .eveuniverse import EsiNameLocalization, EveSolarSystem
+from .eveuniverse import (
+    EsiNameLocalization,
+    EveMoon,
+    EvePlanet,
+    EveSolarSystem,
+    EveSovereigntyMap,
+    EveSpaceType,
+    EveType,
+)
 
 logger = LoggerAddTag(get_extension_logger(__name__), __title__)
 
@@ -48,13 +57,10 @@ class StructureTag(models.Model):
         # TODO: add localization
 
     SPACE_TYPE_MAP = {
-        EveSolarSystem.TYPE_HIGHSEC: {"name": NAME_HIGHSEC_TAG, "style": Style.GREEN},
-        EveSolarSystem.TYPE_LOWSEC: {"name": NAME_LOWSEC_TAG, "style": Style.ORANGE},
-        EveSolarSystem.TYPE_NULLSEC: {"name": NAME_NULLSEC_TAG, "style": Style.RED},
-        EveSolarSystem.TYPE_W_SPACE: {
-            "name": NAME_W_SPACE_TAG,
-            "style": Style.LIGHT_BLUE,
-        },
+        EveSpaceType.HIGHSEC: {"name": NAME_HIGHSEC_TAG, "style": Style.GREEN},
+        EveSpaceType.LOWSEC: {"name": NAME_LOWSEC_TAG, "style": Style.ORANGE},
+        EveSpaceType.NULLSEC: {"name": NAME_NULLSEC_TAG, "style": Style.RED},
+        EveSpaceType.W_SPACE: {"name": NAME_W_SPACE_TAG, "style": Style.LIGHT_BLUE},
     }
 
     name = models.CharField(
@@ -189,41 +195,30 @@ class Structure(models.Model):
     id = models.BigIntegerField(
         primary_key=True, help_text="The Item ID of the structure"
     )
-    owner = models.ForeignKey(
-        "Owner",
-        on_delete=models.CASCADE,
-        related_name="structures",
-        help_text="Corporation that owns the structure",
-    )
-    eve_type = models.ForeignKey(
-        "EveType", on_delete=models.CASCADE, help_text="type of the structure"
-    )
-    name = models.CharField(max_length=255, help_text="The full name of the structure")
-    eve_solar_system = models.ForeignKey("EveSolarSystem", on_delete=models.CASCADE)
-    eve_planet = models.ForeignKey(
-        "EvePlanet",
-        on_delete=models.SET_DEFAULT,
-        null=True,
-        default=None,
-        blank=True,
-        help_text="Planet next to this structure - if any",
+
+    created_at = models.DateTimeField(
+        default=now,
+        help_text="date this structure was received from ESI for the first time",
     )
     eve_moon = models.ForeignKey(
-        "EveMoon",
+        EveMoon,
         on_delete=models.SET_DEFAULT,
         null=True,
         default=None,
         blank=True,
         help_text="Moon next to this structure - if any",
     )
-    position_x = models.FloatField(
-        null=True, default=None, blank=True, help_text="x position in the solar system"
+    eve_planet = models.ForeignKey(
+        EvePlanet,
+        on_delete=models.SET_DEFAULT,
+        null=True,
+        default=None,
+        blank=True,
+        help_text="Planet next to this structure - if any",
     )
-    position_y = models.FloatField(
-        null=True, default=None, blank=True, help_text="y position in the solar system"
-    )
-    position_z = models.FloatField(
-        null=True, default=None, blank=True, help_text="z position in the solar system"
+    eve_solar_system = models.ForeignKey(EveSolarSystem, on_delete=models.CASCADE)
+    eve_type = models.ForeignKey(
+        EveType, on_delete=models.CASCADE, help_text="type of the structure"
     )
     fuel_expires_at = models.DateTimeField(
         null=True,
@@ -231,6 +226,33 @@ class Structure(models.Model):
         blank=True,
         help_text="Date on which the structure will run out of fuel",
     )
+    has_fitting = models.BooleanField(
+        null=True,
+        default=None,
+        blank=True,
+        db_index=True,
+        help_text="bool indicating if the structure has a fitting",
+    )
+    has_core = models.BooleanField(
+        null=True,
+        default=None,
+        blank=True,
+        db_index=True,
+        help_text="bool indicating if the structure has a quantum core",
+    )
+    last_online_at = models.DateTimeField(
+        null=True,
+        default=None,
+        blank=True,
+        help_text="date this structure had any of it's services online",
+    )
+    last_updated_at = models.DateTimeField(
+        null=True,
+        default=None,
+        blank=True,
+        help_text="date this structure was last updated from the EVE server",
+    )
+    name = models.CharField(max_length=255, help_text="The full name of the structure")
     next_reinforce_hour = models.PositiveIntegerField(
         null=True,
         default=None,
@@ -250,6 +272,12 @@ class Structure(models.Model):
             "effect at the time shown by next_reinforce_apply"
         ),
     )
+    owner = models.ForeignKey(
+        "Owner",
+        on_delete=models.CASCADE,
+        related_name="structures",
+        help_text="Corporation that owns the structure",
+    )
     reinforce_hour = models.PositiveIntegerField(
         validators=[MaxValueValidator(23)],
         null=True,
@@ -263,75 +291,90 @@ class Structure(models.Model):
             "is +/- 2 hours centered on the value of this property"
         ),
     )
+    position_x = models.FloatField(
+        null=True, default=None, blank=True, help_text="x position in the solar system"
+    )
+    position_y = models.FloatField(
+        null=True, default=None, blank=True, help_text="y position in the solar system"
+    )
+    position_z = models.FloatField(
+        null=True, default=None, blank=True, help_text="z position in the solar system"
+    )
     state = models.IntegerField(
         choices=State.choices,
         default=State.UNKNOWN,
         blank=True,
         help_text="Current state of the structure",
     )
-    state_timer_start = models.DateTimeField(
-        null=True,
-        default=None,
-        blank=True,
-        help_text="Date at which the structure will move to it’s next state",
-    )
     state_timer_end = models.DateTimeField(
         null=True,
         default=None,
         blank=True,
         help_text="Date at which the structure entered it’s current state",
     )
+    state_timer_start = models.DateTimeField(
+        null=True,
+        default=None,
+        blank=True,
+        help_text="Date at which the structure will move to it’s next state",
+    )
+    tags = models.ManyToManyField(
+        StructureTag,
+        default=None,
+        blank=True,
+        related_name="structures",
+        help_text="List of tags for this structure. ",
+    )
     unanchors_at = models.DateTimeField(
         null=True,
         default=None,
         blank=True,
         help_text="Date at which the structure will unanchor",
     )
-    last_online_at = models.DateTimeField(
-        null=True,
+    webhooks = models.ManyToManyField(
+        "Webhook",
         default=None,
         blank=True,
-        help_text="date this structure had any of it's services online",
-    )
-    tags = models.ManyToManyField(
-        StructureTag,
-        default=None,
-        blank=True,
-        help_text="list of tags for this structure",
-    )
-    last_updated_at = models.DateTimeField(
-        null=True,
-        default=None,
-        blank=True,
-        help_text="date this structure was last updated from the EVE server",
-    )
-    created_at = models.DateTimeField(
-        default=now,
-        help_text="date this structure was received from ESI for the first time",
-    )
-    has_fitting = models.BooleanField(
-        null=True,
-        default=None,
-        blank=True,
-        db_index=True,
-        help_text="bool indicating if the structure has a fitting",
-    )
-    has_core = models.BooleanField(
-        null=True,
-        default=None,
-        blank=True,
-        db_index=True,
-        help_text="bool indicating if the structure has a quantum core",
+        related_name="structures",
+        help_text=(
+            "Webhooks for sending notifications to. "
+            "If any webhook is enabled, these will be used instead of the webhooks "
+            "defined for the respective owner. "
+            "If no webhook is enabled the owner's setting will be used. "
+        ),
     )
 
     objects = StructureManager()
 
     def __str__(self) -> str:
-        return f"{self.id} - {self.eve_solar_system} - {self.name}"
+        if self.is_upwell_structure:
+            try:
+                location_name = self.eve_solar_system.name
+            except AttributeError:
+                location_name = "?"
+        else:
+            location_name = self.location_name
+        return f"{location_name} - {self.name}"
 
     def __repr__(self) -> str:
-        return "{}(id={}, name='{}')".format(
-            self.__class__.__name__, self.id, self.name
+        try:
+            eve_solar_system_name = self.eve_solar_system.name
+        except AttributeError:
+            eve_solar_system_name = ""
+        try:
+            eve_planet_name = self.eve_planet.name
+        except AttributeError:
+            eve_planet_name = ""
+        try:
+            eve_moon_name = self.eve_moon.name
+        except AttributeError:
+            eve_moon_name = ""
+        return (
+            f"{self.__class__.__name__}(id={self.id}, "
+            f"eve_solar_system='{eve_solar_system_name}', "
+            f"eve_planet='{eve_planet_name}', "
+            f"eve_moon='{eve_moon_name}', "
+            f"name='{self.name}')"
         )
 
     def save(self, *args, **kwargs):
@@ -345,15 +388,23 @@ class Structure(models.Model):
 
     @cached_property
     def is_upwell_structure(self) -> bool:
-        return self.eve_type.is_upwell_structure
+        try:
+            return self.eve_type.eve_group.eve_category_id == EveCategoryId.STRUCTURE
+        except AttributeError:
+            logger.warning(
+                "Group %s does not have a category. This is a data error. "
+                "Please update your local SDE data",
+                self.eve_group,
+            )
+            return False
 
     @cached_property
     def is_poco(self) -> bool:
-        return self.eve_type.is_poco
+        return self.eve_type_id == EveTypeId.CUSTOMS_OFFICE
 
     @cached_property
     def is_starbase(self) -> bool:
-        return self.eve_type.is_starbase
+        return starbases.is_starbase(self.eve_type)
 
     @property
     def is_full_power(self) -> bool:
@@ -441,16 +492,25 @@ class Structure(models.Model):
 
     @cached_property
     def owner_has_sov(self) -> bool:
-        return self.eve_solar_system.corporation_has_sov(self.owner.corporation)
+        return EveSovereigntyMap.objects.corporation_has_sov(
+            eve_solar_system=self.eve_solar_system, corporation=self.owner.corporation
+        )
 
     @cached_property
     def location_name(self) -> str:
         """Name of this structures's location."""
-        if self.eve_moon:
+        try:
             return self.eve_moon.name
-        if self.eve_planet:
+        except AttributeError:
+            pass
+        try:
             return self.eve_planet.name
-        return self.eve_solar_system.name
+        except AttributeError:
+            pass
+        try:
+            return self.eve_solar_system.name
+        except AttributeError:
+            return "?"
 
     @cached_property
     def structure_fuel_quantity(self) -> Optional[int]:
@@ -626,14 +686,14 @@ class StructureItem(models.Model):
 
     id = models.BigIntegerField(primary_key=True, help_text="The Eve item ID")
     structure = models.ForeignKey(
-        "Structure",
+        Structure,
         on_delete=models.CASCADE,
         related_name="items",
         help_text="Structure this item is located in",
     )
 
     eve_type = models.ForeignKey(
-        "EveType",
+        EveType,
         on_delete=models.CASCADE,
         help_text="eve type of the item",
         related_name="+",
@@ -676,6 +736,7 @@ class StructureService(EsiNameLocalization, models.Model):
         help_text="Structure this service is installed to",
     )
     name = models.CharField(max_length=100, help_text="Name of the service")
+
     state = models.IntegerField(
         choices=State.choices, help_text="Current state of this service"
     )
@@ -716,9 +777,6 @@ class PocoDetails(models.Model):
             except KeyError:
                 return cls.NONE
 
-    structure = models.OneToOneField(
-        Structure, on_delete=models.CASCADE, related_name="poco_details"
-    )
     alliance_tax_rate = models.FloatField(null=True, default=None)
     allow_access_with_standings = models.BooleanField()
     allow_alliance_access = models.BooleanField()
@@ -732,6 +790,9 @@ class PocoDetails(models.Model):
     standing_level = models.IntegerField(
         choices=StandingLevel.choices, default=StandingLevel.NONE
     )
+    structure = models.OneToOneField(
+        Structure, on_delete=models.CASCADE, related_name="poco_details"
+    )
     terrible_standing_tax_rate = models.FloatField(null=True, default=None)
 
     def __str__(self):
